<?php
class WECR_License{

	function __construct(){
		add_action('admin_menu', array( $this, 'wecr_license_page' ) );
		add_action('admin_init', array( $this, 'register_wecr_option' ) );
		add_action( 'sfn_add_license_field', array( $this, 'add_plugin_license' ), 10, 2 );

		add_action( 'wp_ajax_sfn_activate_key', array( $this, 'activate_license' ) );
		add_action( 'wp_ajax_sfn_deactivate_key', array( $this, 'deactivate_license' ) );

	}


	/**
	 * Adds our license page to the submenu
	 *
	 * @since 1.0
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @uses add_submenu_page()                 Adds a submenu to a WordPress admin menu
	 */
	public function wecr_license_page() {
		add_submenu_page( 'woocommerce', 'SFN License', 'SFN License', 'manage_options', 'sfn-license', 'sfn_license_page' );
	}

	/**
	 * Adds our settings to the option table
	 *
	 * @since 1.0
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @uses register_setting()             Registers the setting with WP
	 */
	public function register_wecr_option() {
		// creates our settings in the options table
		register_setting( 'sfn_license', 'sfn_license', array( $this, 'sanitize_license' ) );
	}

	/**
	 * Sanitizes the license during save
	 *
	 * @since 1.0
	 * @author SFNdesign, Curtis McHale
	 *
	 * @param string        $new            required            New license key
	 * @uses get_option()                                       Returns option from the db given key
	 * @uses update_option()                                    Updates given option
	 */
	public function sanitize_license( $new ) {

		$old = get_option( 'sfn_license' );
		$old = isset( $old['wecr_license'] ) ? $old['wecr_license'] : '';

		if( $old && $old != $new ) {
			$status = get_option( 'sfn_wecr_license_status' );
			$status = null;
			update_option( 'sfn_wecr_license_status', $status ); // new license has been entered, so must reactivate
		}

		return $new;
	}

	/**
	 * Adds the plugin license form
	 *
	 * @since 1.0
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param array         $license        required            All the SFN liceneses
	 * @uses get_option()                                       Returns given option from WP
	 * @uses esc_attr_e()                                       Escapes and echos
	 * @uses wp_nonce_field()                                   Creates a nonce for us
	 */
	public function add_plugin_license( $license ){

		$license = isset( $license['wecr_license'] ) ? $license['wecr_license'] : false;
		$status = get_option( 'sfn_wecr_license_status' );

		if( $status !== false && $status == 'valid' ) {
			$class = 'valid';
		} else {
			$class = 'invalid';
		}

	?>

		<table class="form-table">
			<tbody>
				<tr valign="top">
					<th scope="row" valign="top">
						<?php _e('Easy Content Restriction License Key'); ?>
					</th>
					<td>
						<input id="wecr_license" name="sfn_license[wecr_license]" type="text" class="regular-text" value="<?php esc_attr_e( $license ); ?>" />
						<label class="description" for="sfn_license[wecr_license]"><?php _e('Enter your license key'); ?></label>
					</td>
				</tr>
				<?php if ( false !== $license ) { ?>
					<tr valign="top">
						<th scope="row" valign="top">
							<?php _e('Activate License'); ?>
						</th>
						<td>
							<input type="submit" class="button-secondary edd_license_deactivate <?php echo esc_attr( $class ); ?>" name="edd_license_deactivate" value="<?php _e('Deactivate License'); ?>"/>
							<input type="submit" class="button-secondary edd_license_activate <?php echo esc_attr( $class ); ?>" name="edd_license_activate" value="<?php _e('Activate License'); ?>"/>
							<p class="sfn-message"></p>
						</td>
					</tr>
				<?php } ?>
			</tbody>
		</table>

	<?php
	}

	/**
	 * License deactivation routine
	 *
	 * @since 1.0
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @uses check_admin_referer()              Checks the given nonce param
	 * @uses get_option()                       Returns option given string
	 * @uses home_url()                         echos home url of the site
	 * @uses wp_remote_get()                    Makes http get request
	 * @uses add_query_arg()                    Adds query args to a URL
	 * @uses is_wp_error()                      Returns true if object is WP_ERROR
	 * @uses wp_remote_retriev_body()           Gets the HTML body of a page retrieved
	 * @uses delete_option()                    Deletes given option
	 */
	public function deactivate_license() {

		check_ajax_referer( 'sfn_license_nonce', 'security' );

		// retrieve the license from the database
		$license = get_option( 'sfn_license' );
		$license = trim($license['wecr_license'] );
		$message = '';

		// data to send in our API request
		$api_params = array(
			'edd_action' => 'deactivate_license',
			'license'    => $license,
			'item_name'  => urlencode( WECR_PLUGIN_NAME ), // the name of our product in EDD
			'url'        => home_url()
		);

		// Call the custom API.
		$response = wp_remote_post( SFN_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ){

			// make sure the response came back okay
			if ( is_wp_error( $response ) ) {
				$message = $response->get_error_message();
			} else {
				$message = __( 'An error occurred, please content support@sfndesign.ca' );
			}

		} // is_wp_error $response || wp_remote

		// decode the license data
		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// $license_data->license will be either "deactivated" or "failed"
		if( $license_data->license == 'deactivated' ) {
			delete_option( 'sfn_wecr_license_status' );
			$message = 'License deactivated';
		}

		$data = array(
			'license_data' => $license_data,
			'message' => $message,
		);

		wp_send_json_success( $data );

	}

	/**
	 * License activation routine
	 *
	 * @since 1.0
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @uses check_admin_referer()              Checks the given nonce param
	 * @uses get_option()                       Returns option given string
	 * @uses home_url()                         echos home url of the site
	 * @uses wp_remote_get()                    Makes http get request
	 * @uses add_query_arg()                    Adds query args to a URL
	 * @uses is_wp_error()                      Returns true if object is WP_ERROR
	 * @uses wp_remote_retriev_body()           Gets the HTML body of a page retrieved
	 * @uses update_option()                    updates given option
	 */
	function activate_license() {

		check_ajax_referer( 'sfn_license_nonce', 'security' );

		// retrieve the license from the database
		$license = get_option( 'sfn_license' );
		$license = trim( $license['wecr_license'] );

		// data to send in our API request
		$api_params = array(
			'edd_action' => 'activate_license',
			'license'    => $license,
			'item_name'  => urlencode( WECR_PLUGIN_NAME ), // the name of our product in EDD
			'url'        => home_url()
		);

		// Call the custom API.
		$response = wp_remote_post( SFN_STORE_URL , array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		if ( is_wp_error( $resopnse ) || 200 !== wp_remote_retrieve_response_code( $response ) ){

			// make sure the response came back okay
			if ( is_wp_error( $response ) ){
				$message = $response->get_error_message();
			} else {
				$message = __( 'An error occurred, please try again or contact support: support@sfndesign.ca' );
			}

		} else {

			// decode the license data
			$license_data = json_decode( wp_remote_retrieve_body( $response ) );

			if ( false === $license_data->success ){

				switch( $license_data->error ) {

					case 'expired' :

						$message = sprintf(
							__( 'Your license key expired on %s.' ),
							date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
						);
						break;

					case 'disabled' :
					case 'revoked' :

						$message = __( 'Your license key has been disabled.' );
						break;

					case 'missing' :

						$message = __( 'Invalid license.' );
						break;

					case 'invalid' :
					case 'site_inactive' :

						$message = __( 'Your license is not active for this URL.' );
						break;

					case 'item_name_mismatch' :

						$message = sprintf( __( 'This appears to be an invalid license key for %s.' ), WECR_PLUGIN_NAME );
						break;

					case 'no_activations_left':

						$message = __( 'Your license key has reached its activation limit.' );
						break;

					default :

						$message = __( 'An error occurred, please try again.' );
						break;
				} // switch

			} else { // false == license_data

				// updating the plugin status
				update_option( 'sfn_wecr_license_status', $license_data->license );
				$message = 'License activated';

			} // false === license data

		}

		$data = array(
			'license_data' => $license_data,
			'message' => $message,
		);

		wp_send_json_success( $data );
	}

} // WECR_License

new WECR_license();

if ( ! function_exists( 'sfn_license_page' ) ){

	/**
	 * Base for plugin activations pages
	 *
	 * @since 1.0
	 * @author SFNdesign, Curtis McHale
	 *
	 * @uses get_option()           Returns given option from the DB
	 * @uses settings_fields()      silly settings api stuff
	 * @uses submit_button()        Gives us a settings button out of the WP settings api
	 */
	function sfn_license_page(){

		$license = get_option( 'sfn_license' );

		?>
		<div class="wrap">
			<h2><?php _e('SFN License'); ?></h2>
			<form id="sfn-license-form" method="post" action="options.php">

				<?php settings_fields('sfn_license'); ?>

				<?php do_action( 'sfn_add_license_field', $license ); ?>

				<?php submit_button(); ?>

			</form>
		<?php

	} // sfn_license_page

} // if function_exists
